local SchamzillasFightStats = _G.SchamzillasFightStats

local DefaultDB = {

	guiData = {
		isVisible = true,
		isMinimized = false,
		relativePoint = "TOPLEFT",
		x = 200,
		y = -200,
		width = 420,
		height = 400
	}

}

local function newRingBuffer(size)

	local self = {}

	local length = size
	local head = 0
	local tail = 0
	local pointer = 0
	local list = {}

	self.clear = function()
		head = 0
		tail = 0
		pointer = 0
		list = {}
	end

	self.movePointerForward = function()

		if pointer ~= head then
			pointer = ( pointer + 1 ) % length
		end

	end

	self.movePointerBackwards = function()

		if pointer ~= tail then

			if pointer == 0 then
				pointer = length - 1
			else
				pointer = pointer - 1
			end

		end

	end

	self.getCurrentElement = function()
		return list[pointer]
	end

	self.getNextElement = function()
		self.movePointerForward()
		return self.getCurrentElement()
	end

	self.getPreviousElement = function()
		self.movePointerBackwards()
		return self.getCurrentElement()
	end

	self.remove = function()

		local val = ( tail + 1 ) % length

		if val == pointer then
			self.movePointerForward()
		end

		if val == head then
			self.clear()
		else
			tail = val
		end

	end

	self.add = function(object)

		head = ( head + 1 ) % length
		self.movePointerForward()

		if head == tail then
			tail = ( tail + 1 ) % length
		end

		list[head] = object

	end

	return self

end

local fightHistory = newRingBuffer(50)

local function addFightToHistoryList(fightTable)
	fightHistory.add(fightTable)
end

local function loadFightFromHistoryList(delta)

	if delta < 0 then
		return fightHistory.getPreviousElement()
	elseif delta > 0 then
		return fightHistory.getNextElement()
	else
		return fightHistory.getCurrentElement()
	end

end



function SchamzillasFightStats.DataController:addFightToHistory(fightTable)
	addFightToHistoryList(fightTable)
end

function SchamzillasFightStats.DataController:loadFightFromHistory(delta)
	return loadFightFromHistoryList(delta)
end

function SchamzillasFightStats.DataController:loadGuiPosition()

	if SchamzillasFightStatsCharDB.guiData.relativePoint == nil then
		SchamzillasFightStatsCharDB.guiData.relativePoint = DefaultDB.guiData.relativePoint
	end

	if SchamzillasFightStatsCharDB.guiData.x == nil then
		SchamzillasFightStatsCharDB.guiData.x = DefaultDB.guiData.x
	end

	if SchamzillasFightStatsCharDB.guiData.y == nil then
		SchamzillasFightStatsCharDB.guiData.y = DefaultDB.guiData.y
	end

	return SchamzillasFightStatsCharDB.guiData.relativePoint, SchamzillasFightStatsCharDB.guiData.x, SchamzillasFightStatsCharDB.guiData.y

end

function SchamzillasFightStats.DataController:loadGuiSize()

	if SchamzillasFightStatsCharDB.guiData.width == nil then
		SchamzillasFightStatsCharDB.guiData.width = DefaultDB.guiData.width
	end

	if SchamzillasFightStatsCharDB.guiData.height == nil then
		SchamzillasFightStatsCharDB.guiData.height = DefaultDB.guiData.height
	end

	return SchamzillasFightStatsCharDB.guiData.width, SchamzillasFightStatsCharDB.guiData.height

end

function SchamzillasFightStats.DataController:loadVisibleStatus()

	if SchamzillasFightStatsCharDB.guiData.isVisible == nil then
		SchamzillasFightStatsCharDB.guiData.isVisible = DefaultDB.guiData.isVisible
	end

	return SchamzillasFightStatsCharDB.guiData.isVisible

end

function SchamzillasFightStats.DataController:loadMinimizeStatus()

	if SchamzillasFightStatsCharDB.guiData.isMinimized == nil then
		SchamzillasFightStatsCharDB.guiData.isMinimized = DefaultDB.guiData.isMinimized
	end

	return SchamzillasFightStatsCharDB.guiData.isMinimized

end

function SchamzillasFightStats.DataController:saveGuiPosition(relativePoint, x, y)
	SchamzillasFightStatsCharDB.guiData.relativePoint = relativePoint
	SchamzillasFightStatsCharDB.guiData.x = x
	SchamzillasFightStatsCharDB.guiData.y = y
end

function SchamzillasFightStats.DataController:saveGuiSize(width, height)
	SchamzillasFightStatsCharDB.guiData.width = width
	SchamzillasFightStatsCharDB.guiData.height = height
end

function SchamzillasFightStats.DataController:saveVisibleStatus(isVisible)
	SchamzillasFightStatsCharDB.guiData.isVisible = isVisible
end

function SchamzillasFightStats.DataController:saveMinimizeStatus(isMinimized)
	SchamzillasFightStatsCharDB.guiData.isMinimized = isMinimized
end

function SchamzillasFightStats.DataController:loadDefaultTable()

	SchamzillasFightStatsCharDB = {

		guiData = {
			isVisible = true,
			isMinimized = false,
			relativePoint = "TOPLEFT",
			x = 200,
			y = -200,
			width = 420,
			height = 400
		}

	}

end

function SchamzillasFightStats.DataController:OnLoad()

	if type(SchamzillasFightStatsCharDB) ~= "table" then
		SchamzillasFightStats.DataController:loadDefaultTable()
	end

end