local SchamzillasFightStats = _G.SchamzillasFightStats

local function OnCombatStart()
	SchamzillasFightStats.LogicController:startCombat()
end

local function OnEncounterStart(...)
	SchamzillasFightStats.LogicController:setEncounterName(select(2, ...))
	OnCombatStart()
end

local function OnCombatLogEvent(...)
	SchamzillasFightStats.LogicController:combatEvent(...)
end

local function OnUpdate()
	SchamzillasFightStats.LogicController:timerTick()
end

local function OnLoad()
	SchamzillasFightStats.DataController:OnLoad()
	SchamzillasFightStats.LogicController:OnLoad()
end

local function OnCommandLineEntered(msg)
	SchamzillasFightStats.LogicController:executeCommandLine(msg)
end

local function initControllers()

	SchamzillasFightStats.ModuleController:init()

	local numberOfModules = SchamzillasFightStats.ModuleController:getNumberOfModules()
	SchamzillasFightStats.GuiController:init(numberOfModules)

end



local function init()

	initControllers()

	local timer = 0

	SLASH_SCHAMZILLASFIGHTSTATS1 = "/SchamzillasFightStats"
	function SlashCmdList.SCHAMZILLASFIGHTSTATS(msg)
		OnCommandLineEntered(msg)
	end

	SchamzillasFightStats:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	SchamzillasFightStats:RegisterEvent("ADDON_LOADED")
	SchamzillasFightStats:RegisterEvent("PLAYER_REGEN_DISABLED")
	SchamzillasFightStats:RegisterEvent("ENCOUNTER_START")

	SchamzillasFightStats:SetScript("OnEvent", function(self, event, ...)

		if event == "PLAYER_REGEN_DISABLED" then
			OnCombatStart()
		elseif event == "ENCOUNTER_START" then
			OnEncounterStart(...)
		elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
			OnCombatLogEvent(...)
		elseif event == "ADDON_LOADED" and ... == "SchamzillasFightStats" then
			OnLoad()
		end

	end)

	SchamzillasFightStats:SetScript("OnUpdate", function(self, elapsed)

		timer = timer + elapsed

		if timer > 2 then
			OnUpdate()
			timer = 0
		end

	end)



	SchamzillasFightStats.GuiController:getHistoryBackButton():SetScript("OnClick", function()
		SchamzillasFightStats.LogicController:fightHistoryBack()
	end)

	SchamzillasFightStats.GuiController:getHistoryForwardButton():SetScript("OnClick", function()
		SchamzillasFightStats.LogicController:fightHistoryForward()
	end)

	SchamzillasFightStats.GuiController:getTitleBar():SetScript("OnMouseDown", function(self, button)
		if button == "LeftButton" then
			SchamzillasFightStats.LogicController:titleBarMouseDragStart()
		end
	end)

	SchamzillasFightStats.GuiController:getTitleBar():SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" then
			SchamzillasFightStats.LogicController:titleBarMouseDragStop()
		end
	end)

	SchamzillasFightStats.GuiController:getResizeDragpoint():SetScript("OnMouseDown", function(self, button)
		if button == "LeftButton" then
			SchamzillasFightStats.LogicController:resizePointDragStart()
		end
	end)

	SchamzillasFightStats.GuiController:getResizeDragpoint():SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" then
			SchamzillasFightStats.LogicController:resizePointDragStop()
		end
	end)

	SchamzillasFightStats.GuiController:getTitleBar():SetScript("OnHide", function(self)
		SchamzillasFightStats.LogicController:titleBarMouseDragStop()
		SchamzillasFightStats.LogicController:resizePointDragStop()
	end)

	SchamzillasFightStats.GuiController:getShoutButtonSay():SetScript("OnClick", function()
		SchamzillasFightStats.LogicController:printScores("Say")
	end)

	SchamzillasFightStats.GuiController:getShoutButtonIni():SetScript("OnClick", function()
		SchamzillasFightStats.LogicController:printScores("Instance")
	end)

	SchamzillasFightStats.GuiController:getShoutButtonRaid():SetScript("OnClick", function()
		SchamzillasFightStats.LogicController:printScores("Raid")
	end)

	SchamzillasFightStats.GuiController:getClosedButton():SetScript("OnClick", function()
		SchamzillasFightStats.LogicController:closeScoresGui()
	end)

	SchamzillasFightStats.GuiController:getMinimizeButton():SetScript("OnClick", function()
		SchamzillasFightStats.LogicController:minimizeScoresGui()
	end)

	SchamzillasFightStats.GuiController:getRestoreButton():SetScript("OnClick", function()
		SchamzillasFightStats.LogicController:restoreScoresGui()
	end)

end

init()