local SchamzillasFightStats = _G.SchamzillasFightStats

local gui
local rowHeight = 20
local minWidth = 286
local scrollSpeed = 8



local function newButton(parent)

	local color = {r=0.1, g=0.1, b=0.1}

	local highlightColor = {r=0.13, g=0.13, b=0.13}

	local borderTopLeftColor = {r=color.r + 0.1, g=color.g + 0.1, b=color.b + 0.1}
	local borderBottomRightColor = {r=color.r - 0.1, g=color.g - 0.1, b=color.b - 0.1}

	local font = "Fonts\\ARIALN.TTF"
	local fontSize = 12
	local fontColor = {r=0.4, g=0.4, b=1.0}

	local button = CreateFrame("Button", nil, parent)
	button:EnableMouse(true)

		button.bgTexture = button:CreateTexture()
		button.bgTexture:SetPoint("TOPLEFT", 1, -1)
		button.bgTexture:SetPoint("BOTTOMRIGHT", -1, 1)
		button.bgTexture:SetColorTexture(color.r, color.g, color.b, 0.9)

		button.foreground = CreateFrame("Frame", nil, button)
		button.foreground:SetPoint("TOPLEFT", 0, 0)
		button.foreground:SetPoint("BOTTOMRIGHT", 0, 0)

			button.foreground.text = button.foreground:CreateFontString()
			button.foreground.text:SetPoint("TOPLEFT", 1, -1)
			button.foreground.text:SetPoint("BOTTOMRIGHT", -1, 2)
			button.foreground.text:SetFont(font, fontSize)
			button.foreground.text:SetTextColor(fontColor.r, fontColor.g, fontColor.b, 1.0)
			button.foreground.text:SetWordWrap(false)
			button.foreground.text:SetJustifyH("CENTER")
			button.foreground.text:SetJustifyV("MIDDLE")

			button.foreground.texture = button.foreground:CreateTexture()
			button.foreground.texture:SetPoint("TOPLEFT", 2, -2)
			button.foreground.texture:SetPoint("BOTTOMRIGHT", -2, 2)

		button.border = {}

			button.border.topTexture = button:CreateTexture()
			button.border.topTexture:SetPoint("TOPLEFT", 0, 0)
			button.border.topTexture:SetPoint("TOPRIGHT", 0, 0)
			button.border.topTexture:SetHeight(1)
			button.border.topTexture:SetColorTexture(borderTopLeftColor.r, borderTopLeftColor.g, borderTopLeftColor.b, 0.9)

			button.border.leftTexture = button:CreateTexture()
			button.border.leftTexture:SetPoint("TOPLEFT", 0, 0)
			button.border.leftTexture:SetPoint("BOTTOMLEFT", 0, 0)
			button.border.leftTexture:SetHeight(1)
			button.border.leftTexture:SetColorTexture(borderTopLeftColor.r, borderTopLeftColor.g, borderTopLeftColor.b, 0.9)

			button.border.bottomTexture = button:CreateTexture()
			button.border.bottomTexture:SetPoint("BOTTOMLEFT", 0, 0)
			button.border.bottomTexture:SetPoint("BOTTOMRIGHT", 0, 0)
			button.border.bottomTexture:SetHeight(1)
			button.border.bottomTexture:SetColorTexture(borderBottomRightColor.r, borderBottomRightColor.g, borderBottomRightColor.b, 0.9)

			button.border.rightTexture = button:CreateTexture()
			button.border.rightTexture:SetPoint("TOPRIGHT", 0, 0)
			button.border.rightTexture:SetPoint("BOTTOMRIGHT", 0, 0)
			button.border.rightTexture:SetHeight(1)
			button.border.rightTexture:SetColorTexture(borderBottomRightColor.r, borderBottomRightColor.g, borderBottomRightColor.b, 0.9)

	local setTextureColor = function(texture, rgbColorTable)
		texture:SetColorTexture(rgbColorTable.r, rgbColorTable.g, rgbColorTable.b)
	end

	button:SetScript("OnMouseDown", function(self, button)
		if button == "LeftButton" then
			setTextureColor(self.border.topTexture, borderBottomRightColor)
			setTextureColor(self.border.leftTexture, borderBottomRightColor)
			setTextureColor(self.border.bottomTexture, borderTopLeftColor)
			setTextureColor(self.border.rightTexture, borderTopLeftColor)

			local point, relativeTo, relativePoint, x, y = self.foreground:GetPoint(1)
			self.foreground:SetPoint(relativePoint, x,  y - 1)

			point, relativeTo, relativePoint, x, y = self.foreground:GetPoint(2)
			self.foreground:SetPoint(relativePoint, x,  y - 1)
		end
	end)

	button:SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" then
			setTextureColor(self.border.topTexture, borderTopLeftColor)
			setTextureColor(self.border.leftTexture, borderTopLeftColor)
			setTextureColor(self.border.bottomTexture, borderBottomRightColor)
			setTextureColor(self.border.rightTexture, borderBottomRightColor)

			local point, relativeTo, relativePoint, x, y = self.foreground:GetPoint(1)
			self.foreground:SetPoint(relativePoint, x, y + 1)

			point, relativeTo, relativePoint, x, y = self.foreground:GetPoint(2)
			self.foreground:SetPoint(relativePoint, x, y + 1)
		end
	end)

	button:SetScript("OnEnter", function(self, button)
		setTextureColor(self.bgTexture, highlightColor)

	end)

	button:SetScript("OnLeave", function(self, button)
		setTextureColor(self.bgTexture, color)
	end)



	button.setWidth = function(width)
		button:SetWidth(width)
	end

	button.setHeight = function(height)
		button:SetHeight(height)
	end

	button.setText = function(text)
		button.foreground.text:SetFormattedText(text)
	end

	button.setTexture = function(texture)
		button.foreground.texture:SetTexture(texture)
	end

	return button

end

local function newEntry(parent, row)

	local fontSize = 14
	local font = "Fonts\\ARIALN.TTF"
	local tooltipLabelText = ""
	local tooltipText = ""

	local self = CreateFrame("Frame",nil, parent)

	self:SetPoint("TOPLEFT", 0, - ( ( rowHeight - 1 ) * row ) - 1 )
	self:SetPoint("TOPRIGHT", 0, - ( ( rowHeight - 1 ) * row ) - 1 )
	self:SetHeight(rowHeight - 2)
	self:EnableMouse(true)

		self.background = self:CreateTexture()
		self.background:SetColorTexture(0.0, 0.0, 0.0, 0.4)
		self.background:SetAllPoints(self)

		self.fontStringLabel = self:CreateFontString()
		self.fontStringLabel:SetPoint("TOPLEFT", 1, 0 )
		self.fontStringLabel:SetPoint("BOTTOMLEFT", 1, 0 )
		self.fontStringLabel:SetWidth(130)
		self.fontStringLabel:SetFont(font, fontSize)
		self.fontStringLabel:SetTextColor(1.0, 1.0, 1.0, 1.0)
		self.fontStringLabel:SetWordWrap(false)
		self.fontStringLabel:SetJustifyH("LEFT")
		self.fontStringLabel:SetJustifyV("MIDDLE")

		self.fontStringPlayerName = self:CreateFontString()
		self.fontStringPlayerName:SetPoint("TOPLEFT", 132, 0 )
		self.fontStringPlayerName:SetPoint("BOTTOMLEFT", 132, 0 )
		self.fontStringPlayerName:SetWidth(100)
		self.fontStringPlayerName:SetFont(font, fontSize)
		self.fontStringPlayerName:SetTextColor(1.0, 1.0, 1.0, 1.0)
		self.fontStringPlayerName:SetWordWrap(false)
		self.fontStringPlayerName:SetJustifyH("CENTER")
		self.fontStringPlayerName:SetJustifyV("MIDDLE")

		self.fontStringAmount = self:CreateFontString()
		self.fontStringAmount:SetPoint("TOPLEFT", 233, 0)
		self.fontStringAmount:SetPoint("BOTTOMLEFT", 233, 0)
		self.fontStringAmount:SetHeight(rowHeight)
		self.fontStringAmount:SetWidth(50)
		self.fontStringAmount:SetFont(font, fontSize - 2)
		self.fontStringAmount:SetTextColor(1.0, 1.0, 1.0, 1.0)
		self.fontStringAmount:SetWordWrap(false)
		self.fontStringAmount:SetJustifyH("RIGHT")
		self.fontStringAmount:SetJustifyV("MIDDLE")

		self.fontStringDescription = self:CreateFontString()
		self.fontStringDescription:SetPoint("TOPLEFT", 288, 0)
		self.fontStringDescription:SetPoint("BOTTOMLEFT", 288, 0)
		self.fontStringDescription:SetPoint("TOPRIGHT", -1, 0)
		self.fontStringDescription:SetFont(font, fontSize - 2)
		self.fontStringDescription:SetTextColor(1.0, 1.0, 1.0, 1.0)
		self.fontStringDescription:SetWordWrap(false)
		self.fontStringDescription:SetJustifyH("LEFT")
		self.fontStringDescription:SetJustifyV("MIDDLE")

	self:SetScript("OnEnter", function()
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:SetText(tooltipLabelText)
		GameTooltip:AddLine(tooltipText, 1, 1, 1)
		GameTooltip:Show()
	end)

	self:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)

	self.setEntryData = function(moduleTable)

		tooltipLabelText = moduleTable.label
		tooltipText = moduleTable.tooltip

		self.fontStringLabel:SetFormattedText(moduleTable.label)
		self.fontStringPlayerName:SetFormattedText(gsub(moduleTable.name, "%-[^|]+", ""))
		self.fontStringPlayerName:SetTextColor(moduleTable.color.r, moduleTable.color.g, moduleTable.color.b, 1.0)
		self.fontStringAmount:SetFormattedText(moduleTable.amount)
		self.fontStringDescription:SetFormattedText(moduleTable.misc)

	end

	return self

end

local function newGui(parent, numberOfEntrys)

	local gui = CreateFrame("Frame", nil, parent)
	gui.numberOfEntrys = numberOfEntrys

	gui:SetClampedToScreen(true)
	gui:SetMovable(true)
	gui:SetResizable(true)
	gui:SetMinResize(minWidth, 2 * rowHeight)

		gui.BackgroundTexture = gui:CreateTexture()
		gui.BackgroundTexture:SetAllPoints(gui)
		gui.BackgroundTexture:SetColorTexture(0.0, 0.0, 0.0, 0.3)

		gui.TitleRegion = CreateFrame("Frame", nil, gui)
		gui.TitleRegion:SetHeight(rowHeight)
		gui.TitleRegion:SetPoint("TOPLEFT", 0, 0)
		gui.TitleRegion:SetPoint("TOPRIGHT", 0, 0)
		gui.TitleRegion:EnableMouse(true)

			gui.TitleRegion.TitleTexture = gui.TitleRegion:CreateTexture()
			gui.TitleRegion.TitleTexture:SetAllPoints(gui.TitleRegion)
			gui.TitleRegion.TitleTexture:SetColorTexture(0.0, 0.0, 0.0, 0.7)

			gui.TitleRegion.TitleFontString = gui.TitleRegion:CreateFontString()
			gui.TitleRegion.TitleFontString:SetWidth(180)
			gui.TitleRegion.TitleFontString:SetHeight(rowHeight)
			gui.TitleRegion.TitleFontString:SetPoint("TOPLEFT", 2, 0)
			gui.TitleRegion.TitleFontString:SetFont("Fonts\\MORPHEUS.TTF", 15)
			gui.TitleRegion.TitleFontString:SetTextColor(0.4, 0.4, 1.0, 1.0)
			gui.TitleRegion.TitleFontString:SetWordWrap(false)
			gui.TitleRegion.TitleFontString:SetJustifyH("LEFT")
			gui.TitleRegion.TitleFontString:SetJustifyV("MIDDLE")
			gui.TitleRegion.TitleFontString:SetFormattedText("Schamzilla's Fight Stats")

			gui.TitleRegion.EncounterNameFontString = gui.TitleRegion:CreateFontString()
			gui.TitleRegion.EncounterNameFontString:SetHeight(rowHeight - 6)
			gui.TitleRegion.EncounterNameFontString:SetPoint("TOPLEFT", 182, -3)
			gui.TitleRegion.EncounterNameFontString:SetPoint("TOPRIGHT", -94, -3)
			gui.TitleRegion.EncounterNameFontString:SetFont("Fonts\\ARIALN.TTF", 12)
			gui.TitleRegion.EncounterNameFontString:SetTextColor(1.0, 1.0, 1.0, 1.0)
			gui.TitleRegion.EncounterNameFontString:SetWordWrap(false)
			gui.TitleRegion.EncounterNameFontString:SetJustifyH("RIGHT")
			gui.TitleRegion.EncounterNameFontString:SetJustifyV("MIDDLE")

			gui.TitleRegion.historyBackButton = newButton(gui.TitleRegion)
			gui.TitleRegion.historyBackButton.setWidth(20)
			gui.TitleRegion.historyBackButton.setHeight(rowHeight - 5)
			gui.TitleRegion.historyBackButton:SetPoint("TOPRIGHT", -71, -3)
			gui.TitleRegion.historyBackButton.setText("<")

			gui.TitleRegion.historyForwardButton = newButton(gui.TitleRegion)
			gui.TitleRegion.historyForwardButton.setWidth(20)
			gui.TitleRegion.historyForwardButton.setHeight(rowHeight - 5)
			gui.TitleRegion.historyForwardButton:SetPoint("TOPRIGHT", -48, -3)
			gui.TitleRegion.historyForwardButton.setText(">")

			gui.TitleRegion.ShoutButton = newButton(gui.TitleRegion)
			gui.TitleRegion.ShoutButton.setWidth(20)
			gui.TitleRegion.ShoutButton.setHeight(rowHeight - 5)
			gui.TitleRegion.ShoutButton:SetPoint("TOPRIGHT", -25, -3)
			gui.TitleRegion.ShoutButton.setTexture("Interface\\AddOns\\SchamzillasFightStats\\textures\\post.tga")

				gui.TitleRegion.ShoutButton.ShoutMenu = CreateFrame("Frame", nil, gui.TitleRegion.ShoutButton)
				gui.TitleRegion.ShoutButton.ShoutMenu:SetWidth(62)
				gui.TitleRegion.ShoutButton.ShoutMenu:SetHeight(57)
				gui.TitleRegion.ShoutButton.ShoutMenu:SetPoint("TOPLEFT", - (gui.TitleRegion.ShoutButton.ShoutMenu:GetWidth() / 2), gui.TitleRegion.ShoutButton.ShoutMenu:GetHeight() + 4)

					gui.TitleRegion.ShoutButton.ShoutMenu.bgTexture = gui.TitleRegion.ShoutButton.ShoutMenu:CreateTexture()
					gui.TitleRegion.ShoutButton.ShoutMenu.bgTexture:SetAllPoints(gui.TitleRegion.ShoutButton.ShoutMenu)
					gui.TitleRegion.ShoutButton.ShoutMenu.bgTexture:SetColorTexture(0.0, 0.0, 0.0, 0.7)

					gui.TitleRegion.ShoutButton.ShoutMenu.SayButton = newButton(gui.TitleRegion.ShoutButton.ShoutMenu)
					gui.TitleRegion.ShoutButton.ShoutMenu.SayButton.setHeight(15)
					gui.TitleRegion.ShoutButton.ShoutMenu.SayButton:SetPoint("TOPLEFT", 4, -4)
					gui.TitleRegion.ShoutButton.ShoutMenu.SayButton:SetPoint("TOPRIGHT", -4, -4)
					gui.TitleRegion.ShoutButton.ShoutMenu.SayButton.setText("Say")

					gui.TitleRegion.ShoutButton.ShoutMenu.IniButton = newButton(gui.TitleRegion.ShoutButton.ShoutMenu)
					gui.TitleRegion.ShoutButton.ShoutMenu.IniButton.setHeight(15)
					gui.TitleRegion.ShoutButton.ShoutMenu.IniButton:SetPoint("TOPLEFT", 4, -21)
					gui.TitleRegion.ShoutButton.ShoutMenu.IniButton:SetPoint("TOPRIGHT", -4, -21)
					gui.TitleRegion.ShoutButton.ShoutMenu.IniButton.setText("Instance")

					gui.TitleRegion.ShoutButton.ShoutMenu.RaidButton = newButton(gui.TitleRegion.ShoutButton.ShoutMenu)
					gui.TitleRegion.ShoutButton.ShoutMenu.RaidButton.setHeight(15)
					gui.TitleRegion.ShoutButton.ShoutMenu.RaidButton:SetPoint("TOPLEFT", 4, -38)
					gui.TitleRegion.ShoutButton.ShoutMenu.RaidButton:SetPoint("TOPRIGHT", -4, -38)
					gui.TitleRegion.ShoutButton.ShoutMenu.RaidButton.setText("Raid/Grp")

				gui.TitleRegion.ShoutButton.ShoutMenu:Hide()

			gui.TitleRegion.MinimizeButton = newButton(gui.TitleRegion)
			gui.TitleRegion.MinimizeButton.setWidth(20)
			gui.TitleRegion.MinimizeButton.setHeight(rowHeight - 5)
			gui.TitleRegion.MinimizeButton:SetPoint("TOPRIGHT", -3, -3)
			gui.TitleRegion.MinimizeButton.setTexture("Interface\\AddOns\\SchamzillasFightStats\\textures\\minimize.tga")

			gui.TitleRegion.CloseButton = newButton(gui.TitleRegion)
			gui.TitleRegion.CloseButton.setWidth(20)
			gui.TitleRegion.CloseButton.setHeight(rowHeight - 5)
			gui.TitleRegion.CloseButton:SetPoint("TOPRIGHT", -26, -3)
			gui.TitleRegion.CloseButton.setTexture("Interface\\AddOns\\SchamzillasFightStats\\textures\\close.tga")
			gui.TitleRegion.CloseButton:Hide()

			gui.TitleRegion.RestoreButton = newButton(gui.TitleRegion)
			gui.TitleRegion.RestoreButton.setWidth(20)
			gui.TitleRegion.RestoreButton.setHeight(rowHeight - 5)
			gui.TitleRegion.RestoreButton:SetPoint("TOPRIGHT", -3, -3)
			gui.TitleRegion.RestoreButton.setTexture("Interface\\AddOns\\SchamzillasFightStats\\textures\\restore.tga")
			gui.TitleRegion.RestoreButton:Hide()

		gui.EntryRegion = CreateFrame("ScrollFrame", nil, gui)
		gui.EntryRegion:SetPoint("TOPLEFT", 0, -20)
		gui.EntryRegion:SetPoint("BOTTOMRIGHT", 0, 0)
		gui.EntryRegion:SetFrameLevel(10)
		gui.EntryRegion:EnableMouseWheel(true)

			gui.EntryRegion.ContentFrame = CreateFrame("Frame", nil, gui.EntryRegion)
			gui.EntryRegion.ContentFrame:SetHeight(20 * gui.numberOfEntrys)
			gui.EntryRegion.ContentFrame:SetFrameLevel(15)

				gui.EntryRegion.ContentFrame.EntryList = {}

				for i=0, gui.numberOfEntrys - 1 do
					gui.EntryRegion.ContentFrame.EntryList[i] = newEntry(gui.EntryRegion.ContentFrame, i)
					gui.EntryRegion.ContentFrame.EntryList[i]:SetFrameLevel(20)
				end

			gui.EntryRegion:SetScrollChild(gui.EntryRegion.ContentFrame)

		gui.ResizeDragpoint = CreateFrame("Button", nil, gui)
		gui.ResizeDragpoint:SetWidth(rowHeight)
		gui.ResizeDragpoint:SetHeight(rowHeight)
		gui.ResizeDragpoint:SetPoint("BOTTOMRIGHT", 0, 0)
		gui.ResizeDragpoint:SetFrameLevel(25)
		gui.ResizeDragpoint:EnableMouse(true)

		gui.ResizeDragpoint.Texture = gui.ResizeDragpoint:CreateTexture()
		gui.ResizeDragpoint.Texture:SetTexture("Interface\\AddOns\\SchamzillasFightStats\\textures\\resizecorner.tga")
		gui.ResizeDragpoint.Texture:SetPoint("TOPLEFT", 2, -2)
		gui.ResizeDragpoint.Texture:SetPoint("BOTTOMRIGHT", -2, 2)



	local setScroll = function(delta)

		local scroll = gui.EntryRegion:GetVerticalScroll() - scrollSpeed * delta

		if scroll > gui.EntryRegion:GetScrollChild():GetHeight() - gui.EntryRegion:GetHeight() then
			scroll = gui.EntryRegion:GetScrollChild():GetHeight()- gui.EntryRegion:GetHeight()
		end

		if scroll < 0 then
			scroll = 0
		end

		gui.EntryRegion:SetVerticalScroll(scroll)

	end

	local changeSize = function(width, height)

		gui:SetWidth(width)
		gui:SetHeight(height)

		gui.EntryRegion.ContentFrame:SetWidth(width)

		if width < 320 then
			gui.TitleRegion.EncounterNameFontString:Hide()
		else
			gui.TitleRegion.EncounterNameFontString:Show()
		end

		setScroll(0)

	end

	local showHideShoutMenu = function()

		if gui.TitleRegion.ShoutButton.ShoutMenu:IsVisible() then
			gui.TitleRegion.ShoutButton.ShoutMenu:Hide()
		else
			gui.TitleRegion.ShoutButton.ShoutMenu:Show()
		end

	end

	local calculateAnchorPointCorrection = function(relativPoint, x, y)

		local width = gui:GetWidth()
		local height = gui:GetHeight()

		local switchTable = {
			TOPLEFT = 		function() return 0, 						0 end,
			TOP = 			function() return minWidth / 2 - width / 2, 0 end,
			TOPRIGHT = 		function() return minWidth - width, 		0 end,
			LEFT = 			function() return 0, 						height / 2 - rowHeight / 2 end,
			CENTER = 		function() return minWidth / 2 - width / 2, height / 2 - rowHeight / 2 end,
			RIGHT = 		function() return minWidth - width, 		height / 2 - rowHeight / 2 end,
			BOTTOMLEFT = 	function() return 0, 						height - rowHeight end,
			BOTTOM = 		function() return minWidth / 2 - width / 2, height - rowHeight end,
			BOTTOMRIGHT = 	function() return minWidth - width, 		height - rowHeight end
		}

		return switchTable[relativPoint]()

	end

	gui:SetScript("OnSizeChanged", function(self, width, height)
		changeSize(width, height)
	end)

	gui.EntryRegion:SetScript("OnMouseWheel", function(self, delta)
		setScroll(delta)
	end)

	gui.TitleRegion.ShoutButton:SetScript("OnClick", function()
		showHideShoutMenu()
	end)



	gui.updateGuiData = function(fightTable)

		local numberOfRows = 0

		gui.TitleRegion.EncounterNameFontString:SetFormattedText(fightTable.name)

		for i=0, fightTable.length - 1 do

			gui.EntryRegion.ContentFrame.EntryList[i]:Hide()

			if fightTable[i].name ~= "-" then
				gui.EntryRegion.ContentFrame.EntryList[numberOfRows].setEntryData(fightTable[i])
				gui.EntryRegion.ContentFrame.EntryList[numberOfRows]:SetFrameLevel(20)
				gui.EntryRegion.ContentFrame.EntryList[numberOfRows]:Show()
				numberOfRows = numberOfRows + 1
			end

		end

		gui.EntryRegion.ContentFrame:SetHeight(20 * numberOfRows)
		gui.ResizeDragpoint:SetFrameLevel(25)
		setScroll(0)

	end

	gui.minimize = function()

		local anchorPoint, relativeTo, relativePoint, x, y = gui:GetPoint()
		local xC, yC = calculateAnchorPointCorrection(relativePoint, x, y)

		gui.TitleRegion.ShoutButton.ShoutMenu:Hide()
		gui.TitleRegion.ShoutButton:Hide()
		gui.TitleRegion.MinimizeButton:Hide()

		gui.TitleRegion.RestoreButton:Show()
		gui.TitleRegion.CloseButton:Show()

		gui.EntryRegion:Hide()
		gui.EntryRegion:SetParent(nil)
		gui.ResizeDragpoint:Hide()
		gui.ResizeDragpoint:SetParent(nil)

		gui:SetWidth(minWidth)
		gui:SetHeight(rowHeight)

		gui:ClearAllPoints()
		gui:SetPoint(relativePoint, x + xC, y + yC)

	end

	gui.restore = function(width, height)

		local anchorPoint, relativeTo, relativePoint, x, y = gui:GetPoint()

		gui.TitleRegion.RestoreButton:Hide()
		gui.TitleRegion.CloseButton:Hide()

		gui.TitleRegion.ShoutButton:Show()
		gui.TitleRegion.MinimizeButton:Show()

		gui.EntryRegion:SetParent(gui)
		gui.EntryRegion:Show()
		gui.ResizeDragpoint:SetParent(gui)
		gui.ResizeDragpoint:Show()

		gui:SetWidth(width)
		gui:SetHeight(height)

		local xC, yC = calculateAnchorPointCorrection(relativePoint, x, y)
		gui:ClearAllPoints()
		gui:SetPoint(relativePoint, x - xC, y - yC)

	end

	gui.setSize = function(width, height)
		changeSize(width, height)
	end



	gui.hideShoutButtons = function()
		gui.TitleRegion.ShoutButton.ShoutMenu:Hide()
	end

	gui.getHistoryBackButton = function()
		return gui.TitleRegion.historyBackButton
	end

	gui.getHistoryForwardButton = function()
		return gui.TitleRegion.historyForwardButton
	end

	gui.getResizeDragpoint = function()
		return gui.ResizeDragpoint
	end

	gui.getShoutButtonSay = function()
		return gui.TitleRegion.ShoutButton.ShoutMenu.SayButton
	end

	gui.getShoutButtonIni = function()
		return gui.TitleRegion.ShoutButton.ShoutMenu.IniButton
	end

	gui.getShoutButtonRaid = function()
		return gui.TitleRegion.ShoutButton.ShoutMenu.RaidButton
	end

	gui.getCloseButton = function()
		return gui.TitleRegion.CloseButton
	end

	gui.getMinimizeButton = function()
		return gui.TitleRegion.MinimizeButton
	end

	gui.getRestoreButton = function()
		return gui.TitleRegion.RestoreButton
	end

	gui.getTitleBar = function()
		return gui.TitleRegion
	end

	return gui

end



local function printClosedMessage()

	print(" ")

	local msg = "--- Schamzilla's Fight Stats"
	print(msg)

	msg = "Gui was closed. To open Gui type:"
	print(msg)

	msg = "/SchamzillasFightStats show"
	print(msg)

	print(" ")

end

local function printScores(chatType, fightTable)

	gui.hideShoutButtons()

	local msg

	SendChatMessage(" ", chatType)

	msg = "--- Schamzilla's Fight Stats - " .. fightTable.name
	SendChatMessage(msg, chatType)

	for i=0, fightTable.length - 1 do

		if fightTable[i].name ~= "-" then

			msg = fightTable[i].label .. ": " .. gsub(fightTable[i].name, "%-[^|]+", "") .. " (" .. fightTable[i].amount .. ")"
			SendChatMessage(msg, chatType)

		end

	end

	SendChatMessage(" ", chatType)

end

local function init(numberOfModules, moduleLabelList)
	gui = newGui(UIParent, numberOfModules)
end



function SchamzillasFightStats.GuiController:getHistoryBackButton()
	return gui.getHistoryBackButton()
end

function SchamzillasFightStats.GuiController:getHistoryForwardButton()
	return gui.getHistoryForwardButton()
end

function SchamzillasFightStats.GuiController:getShoutButtonSay()
	return gui.getShoutButtonSay()
end

function SchamzillasFightStats.GuiController:getShoutButtonIni()
	return gui.getShoutButtonIni()
end

function SchamzillasFightStats.GuiController:getShoutButtonRaid()
	return gui.getShoutButtonRaid()
end

function SchamzillasFightStats.GuiController:getClosedButton()
	return gui.getCloseButton()
end

function SchamzillasFightStats.GuiController:getMinimizeButton()
	return gui.getMinimizeButton()
end

function SchamzillasFightStats.GuiController:getRestoreButton()
	return gui.getRestoreButton()
end

function SchamzillasFightStats.GuiController:getResizeDragpoint()
	return gui.getResizeDragpoint()
end

function SchamzillasFightStats.GuiController:getTitleBar()
	return gui.getTitleBar()
end



function SchamzillasFightStats.GuiController:titleBarMouseDragStart()
	if not gui.isMoving then
		gui:StartMoving()
		gui.isMoving = true
	end
end

function SchamzillasFightStats.GuiController:titleBarMouseDragStop()
	if gui.isMoving then
		gui:StopMovingOrSizing()
		gui.isMoving = false
	end
end

function SchamzillasFightStats.GuiController:resizePointDragStart()
	if not gui.isResizing then
		gui:StartSizing()
		gui.isResizing = true
	end
end

function SchamzillasFightStats.GuiController:resizePointDragStop()
	if gui.isResizing then
		gui:StopMovingOrSizing()
		gui.isResizing = false
	end
end

function SchamzillasFightStats.GuiController:getGuiPosition()
	local anchorPoint, relativeTo, relativePoint, x, y = gui:GetPoint()
	return relativePoint, x, y
end

function SchamzillasFightStats.GuiController:getGuiSize()
	return gui:GetWidth(), gui:GetHeight()
end

function SchamzillasFightStats.GuiController:setGuiPosition(relativePoint, x, y)
	gui:ClearAllPoints()
	gui:SetPoint(relativePoint, x, y)
end

function SchamzillasFightStats.GuiController:setGuiSize(width, height)
	gui.setSize(width, height)
end

function SchamzillasFightStats.GuiController:updateScoresGuiData(fightTable)
	gui.updateGuiData(fightTable)
end

function SchamzillasFightStats.GuiController:showScoresGui(width, height)
	gui:SetParent(UIParent)
	gui:Show()
end

function SchamzillasFightStats.GuiController:hideScoresGui()
	printClosedMessage()

	gui:Hide()
	gui:SetParent(nil)
end

function SchamzillasFightStats.GuiController:minimizeScoresGui()
	gui.minimize()
end

function SchamzillasFightStats.GuiController:restoreScoresGui(width, height)
	gui.restore(width, height)
end

function SchamzillasFightStats.GuiController:printScores(chatType, fightTable)
	printScores(chatType, fightTable)
end

function SchamzillasFightStats.GuiController:printCommandlineOptions()

	local msg

	print(" ")

	msg = "--- Schamzilla's Fight Stats"
	print(msg)

	msg = "Usage:"
	print(msg)

	msg = "/SchamzillasFightStats [show, hide, clear]"
	print(msg)

	print(" ")

end

function SchamzillasFightStats.GuiController:init(numberOfModules)
	init(numberOfModules)
end