local SchamzillasFightStats = _G.SchamzillasFightStats

local isInCombat
local encounterName = ""

local function getTimestamp()
	local date = date("*t")
	return string.format("%02d:%02d:%02d", date.hour, date.min, date.sec)
end

local function startCombat()

	local isVisible = SchamzillasFightStats.DataController:loadVisibleStatus()

	if not isInCombat and isVisible then

		if encounterName == "" then
			encounterName = getTimestamp()
		end

		SchamzillasFightStats.ModuleController:init()
		isInCombat = true

	end

end

local function addCurrentFightToHistory()

	local fightTable = SchamzillasFightStats.ModuleController:getFightTable()

	fightTable.name = encounterName
	encounterName = ""

	SchamzillasFightStats.DataController:addFightToHistory(fightTable)

end

local function updateScoresGuiData(fightTable)
	SchamzillasFightStats.GuiController:updateScoresGuiData(fightTable)
end

local function fightHistoryBack()
	local fightTable = SchamzillasFightStats.DataController:loadFightFromHistory(-1)
	updateScoresGuiData(fightTable)
end

local function fightHistoryForward()
	local fightTable = SchamzillasFightStats.DataController:loadFightFromHistory(1)
	updateScoresGuiData(fightTable)
end

local function printScores(chatType)

	local fightTable = SchamzillasFightStats.DataController:loadFightFromHistory(0)

	local cT = ""

	if chatType == "Say" then
		cT = "SAY"
	elseif chatType == "Instance" then
		cT = "INSTANCE_CHAT"
	elseif chatType == "Raid" then
		if UnitInRaid("player") ~= nil then
			cT = "RAID"
		else
			cT = "PARTY"
		end
	end

	if cT ~= "" then
		SchamzillasFightStats.GuiController:printScores(cT, fightTable)
	end

end

local function stopCombat()
	isInCombat = false
	addCurrentFightToHistory()

	local fightTable = SchamzillasFightStats.DataController:loadFightFromHistory(0)
	updateScoresGuiData(fightTable)
end

local function getGroupStats()

	local type = "none"
	local count = 1

	if IsInRaid() then
		count = GetNumGroupMembers()
		type = "raid"
	elseif IsInGroup() then
		count = GetNumGroupMembers() - 1
		type = "party"
	end

	return type, count

end

local function isGroupInCombat()

	if UnitAffectingCombat("player") then
		return true
	end

	local type, count = getGroupStats()

	for i = 1, count, 1 do
		if UnitExists(type..i) and UnitAffectingCombat(type..i) then
			return true
		end
	end

	return false

end

local function setGuiPosition()
	local relativePoint, x, y = SchamzillasFightStats.DataController:loadGuiPosition()
	SchamzillasFightStats.GuiController:setGuiPosition(relativePoint, x, y)
end

local function setGuiSize()
	local width, height = SchamzillasFightStats.DataController:loadGuiSize()
	SchamzillasFightStats.GuiController:setGuiSize(width, height)
end

local function showGui()

	SchamzillasFightStats.DataController:saveVisibleStatus(true)

	setGuiSize()
	setGuiPosition()

	SchamzillasFightStats.GuiController:showScoresGui()

	local isMinimized = SchamzillasFightStats.DataController:loadMinimizeStatus()

	if isMinimized then
		SchamzillasFightStats.GuiController:minimizeScoresGui()
		setGuiPosition()
	end

end

local function restoreGui()
	SchamzillasFightStats.DataController:saveMinimizeStatus(false)

	local width, height = SchamzillasFightStats.DataController:loadGuiSize()
	SchamzillasFightStats.GuiController:restoreScoresGui(width, height)
end

local function saveGuiSize()

	local isMinimized = SchamzillasFightStats.DataController:loadMinimizeStatus()

	if not isMinimized then
		local width, height = SchamzillasFightStats.GuiController:getGuiSize()
		SchamzillasFightStats.DataController:saveGuiSize(width, height)
	end

end

local function saveGuiPosition()
	local relativePoint, x, y = SchamzillasFightStats.GuiController:getGuiPosition()
	SchamzillasFightStats.DataController:saveGuiPosition(relativePoint, x, y)
end

local function showGuiModuleInformation()

	local fightTable = SchamzillasFightStats.ModuleController:getFightTable()

	fightTable.name = "all modules ready"

	for i=0, fightTable.length - 1 do
		fightTable[i].name = "...0k"
		fightTable[i].amount = ""
		fightTable[i].misc = fightTable[i].tooltip
	end

	updateScoresGuiData(fightTable)

end


function SchamzillasFightStats.LogicController:executeCommandLine(msg)

	if msg == "show" then
		showGui()
	elseif msg == "hide" then
		SchamzillasFightStats.GuiController:closeScoresGui()
	elseif msg == "clear" then
		SchamzillasFightStats.DataController:loadDefaultTable()
		showGui()
	else
		SchamzillasFightStats.GuiController:printCommandlineOptions()
	end

end

function SchamzillasFightStats.LogicController:fightHistoryBack()
	fightHistoryBack()
end

function SchamzillasFightStats.LogicController:fightHistoryForward()
	fightHistoryForward()
end

function SchamzillasFightStats.LogicController:titleBarMouseDragStart()
	SchamzillasFightStats.GuiController:titleBarMouseDragStart()
end

function SchamzillasFightStats.LogicController:titleBarMouseDragStop()
	SchamzillasFightStats.GuiController:titleBarMouseDragStop()
	saveGuiPosition()
end

function SchamzillasFightStats.LogicController:resizePointDragStart()
	SchamzillasFightStats.GuiController:resizePointDragStart()
end

function SchamzillasFightStats.LogicController:resizePointDragStop()
	SchamzillasFightStats.GuiController:resizePointDragStop()
	saveGuiSize()
end

function SchamzillasFightStats.LogicController:printScores(chatType)
	printScores(chatType)
end

function SchamzillasFightStats.LogicController:closeScoresGui()
	restoreGui()
	SchamzillasFightStats.DataController:saveVisibleStatus(false)
	SchamzillasFightStats.GuiController:hideScoresGui()
end

function SchamzillasFightStats.LogicController:minimizeScoresGui()
	SchamzillasFightStats.DataController:saveMinimizeStatus(true)
	SchamzillasFightStats.GuiController:minimizeScoresGui()
end

function SchamzillasFightStats.LogicController:restoreScoresGui()
	restoreGui()
end

function SchamzillasFightStats.LogicController:startCombat()
	startCombat()
end

function SchamzillasFightStats.LogicController:setEncounterName(name)
	encounterName = name
end

function SchamzillasFightStats.LogicController:combatEvent(...)

	if isInCombat then
		SchamzillasFightStats.ModuleController:analyzeData(...)
	end

end

function SchamzillasFightStats.LogicController:timerTick()

	if isInCombat then

		if not isGroupInCombat() then
			stopCombat()
		end

	end

end

function SchamzillasFightStats.LogicController:OnLoad()

	isInCombat = UnitAffectingCombat("player")
	combatStartTime = getTimestamp()

	local isVisible = SchamzillasFightStats.DataController:loadVisibleStatus()

	if isVisible then
		showGui()
	end

	addCurrentFightToHistory()

	showGuiModuleInformation()

end