local SchamzillasFightStats = _G.SchamzillasFightStats

local numberOfModules = 0
local moduleList = {}

local function formatNumber(number)

	local n = number

	if n >= 1000000000 then
		n = string.format("%.01f", (n / 1000000000)) .. "B"
	elseif n >= 1000000 then
		n = string.format("%.01f", (n / 1000000)) .. "M"
	elseif n >= 1000 then
		n = string.format("%.01f", (n / 1000)) .. "K"
	else
		n = string.format("%.0f", n)
	end

	return n

end

local function registerModule(module)
	moduleList[numberOfModules] = module
	numberOfModules = numberOfModules + 1
end

local function analyzeData(...)

	for i=0, numberOfModules - 1 do
		moduleList[i]:analyze(...)
	end

end

local function getModuleLabelList()

	local moduleLabelList = {}

	for i=0, numberOfModules - 1 do
		moduleLabelList[i] = moduleList[i].getLabelText()
	end

	return moduleLabelList

end

local function getModuleTable(moduleIndex)
	local score = {}
	score.label = moduleList[moduleIndex].getLabelText()
	score.tooltip = moduleList[moduleIndex].getTooltipText()
	score.name, score.color, score.amount, score.misc = moduleList[moduleIndex]:getScoreInformation()
	score.amount = formatNumber(score.amount)
	return score
end

local function getFightTable()

	local fightTable = {}

	fightTable.name = ""
	fightTable.length = numberOfModules

	for i=0, numberOfModules - 1 do
		fightTable[i] = getModuleTable(i)
	end

	return fightTable

end

local function initModules()

	for i=0, numberOfModules - 1 do
		moduleList[i]:init()
	end

end



function SchamzillasFightStats.ModuleController:registerModule(module)
	registerModule(module)
end

function SchamzillasFightStats.ModuleController:analyzeData(...)
	analyzeData(...)
end

function SchamzillasFightStats.ModuleController:getModuleLabelList()
	return getModuleLabelList()
end

function SchamzillasFightStats.ModuleController:getFightTable()
	return getFightTable()
end

function SchamzillasFightStats.ModuleController:getNumberOfModules()
	return numberOfModules
end

function SchamzillasFightStats.ModuleController:init()
	initModules()
end