local SchamzillasFightStats = _G.SchamzillasFightStats

local timeStamp, eventType, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName

local spellId, spellName, spellSchool, missType, isOffHand, amountMissed, resisted, blocked, absorbed

local checkEvent = {
	SPELL_MISSED = true,
	SPELL_PERIODIC_MISSED = true,
	RANGE_MISSED = true,
	SWING_MISSED = true,
	SPELL_DAMAGE = true,
	SPELL_PERIODIC_DAMAGE = true,
	RANGE_DAMAGE = true,
	SWING_DAMAGE = true
}

local readEvent = {
	SPELL_MISSED = function(...)
		spellId, spellName, spellSchool, missType, isOffHand, amountMissed = select(12, ...)
		return spellName, amountMissed
	end,
	SPELL_PERIODIC_MISSED = function(...)
		spellId, spellName, spellSchool, missType, isOffHand, amountMissed = select(12, ...)
		return spellName, amountMissed
	end,
	RANGE_MISSED = function(...)
		spellId, spellName, spellSchool, missType, isOffHand, amountMissed = select(12, ...)
		return spellName, amountMissed
	end,
	SWING_MISSED = function(...)
		missType, isOffHand, amountMissed = select(12, ...)
		return "Swing", amountMissed
	end,
	SPELL_DAMAGE = function(...)
		spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical = select(12, ...)
		amountMissed = 0
		if resisted ~= nil then amountMissed = amountMissed + resisted end
		if blocked ~= nil then amountMissed = amountMissed	+ blocked end
		if absorbed ~= nil then amountMissed = amountMissed + absorbed end
		return spellName, amountMissed
	end,
	SPELL_PERIODIC_DAMAGE = function(...)
		spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical = select(12, ...)
		amountMissed = 0
		if resisted ~= nil then amountMissed = amountMissed + resisted end
		if blocked ~= nil then amountMissed = amountMissed	+ blocked end
		if absorbed ~= nil then amountMissed = amountMissed + absorbed end
		return spellName, amountMissed
	end,
	RANGE_DAMAGE = function(...)
		spellId, spellName, spellSchool, amount, overkill, school, resisted, blocked, absorbed, critical = select(12, ...)
		amountMissed = 0
		if resisted ~= nil then amountMissed = amountMissed + resisted end
		if blocked ~= nil then amountMissed = amountMissed	+ blocked end
		if absorbed ~= nil then amountMissed = amountMissed + absorbed end
		return spellName, amountMissed
	end,
	SWING_DAMAGE = function(...)
		amount, overkill, school, resisted, blocked, absorbed, critical = select(12, ...)
		amountMissed = 0
		if resisted ~= nil then amountMissed = amountMissed + resisted end
		if blocked ~= nil then amountMissed = amountMissed	+ blocked end
		if absorbed ~= nil then amountMissed = amountMissed + absorbed end
		return "Swing", amountMissed
	end
}

local HighestMiss = {

	mName = "",
	mColor = {},
	mAmount = 0,
	mSpell = "",

	analyze = function(self, ...)

		timeStamp, eventType, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName = ...

		if checkEvent[eventType] ~= nil and UnitIsPlayer(sourceName) and not UnitIsPlayer(destName) then

			spellName, amountMissed = readEvent[eventType](...)

			if amountMissed ~= nil and amountMissed > self.mAmount then
				self.mName = sourceName
				self.mColor = RAID_CLASS_COLORS[select(2,UnitClass(sourceName))]
				self.mAmount = amountMissed
				self.mSpell = spellName
			end

		end

	end,

	getTooltipText = function()
		return 'who wasted most damage in one hit\ndue to resistances, absorb, block, etc.'
	end,

	getLabelText = function()
		return 'Damage Waster'
	end,

	getScoreInformation = function(self)
		return self.mName, self.mColor, self.mAmount, self.mSpell
	end,

	init = function(self)
		self.mName = "-"
		self.mColor = {r = 1.0, g = 1.0, b = 1.0, a = 1.0}
		self.mAmount = 0
		self.mSpell = " "
	end

}

SchamzillasFightStats.ModuleController:registerModule(HighestMiss)